<?php
// Filename : GoogleAuthenticatorClass.php
// Purpose  : Google Authenticator library to enable multi factor authentication
// Author   : Phil Massyn (@massyn)
// Date     : 2018-05-12
// Status   : Alpha - this script is stil being tested.  Production use is NOT encouraged.
//
// Release  : 2018-05-12 - Initial release
//			  2019-11-17 - Closed FORM (was missing) and setup_googleauth.php to enable self-removal of two factor auth for users. Olaf Nöhring, https://datenbank-projekt.de
//
// MIT License
//
// Copyright (c) 2018 Phil Massyn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
// =====================================================================================
class framework_GoogleAuthenticator {
	// When defining TOTP, run this function to generate a secret key, and generate an HTML page that can be returned.  This will allow the user to scan the code, and submit the detail back to you for validation.
	// Input : Username, and title of the application
	public function TOTPsetupPage($name,$title) {
		// generate a secret key
		$secret = $this->createSecret();
		// generate the QR Code URL
		$qr = $this->getQRCodeGoogleUrl($name,$secret,$title);
		$out = "<table border=1>\n";
		$out .= "<tr><th>1</th><td>Open your Google Authenticator app on your mobile phone.  If you do not have the app, you need to install the app.</td></tr>";
		$out .= "<tr><th>2</th><td>Scan the following QR code in your app.<br><img src=\"$qr\"><br>If you cannot see the QR code, you can enter the secret manually<br><b>$secret</b></td></tr>\n";
		$out .= "<tr><th>3</th><td>Enter the code your app returns in this form. <form method=post><input type=input name=otp><input type=hidden name=secret value=\"$secret\"><input type=hidden name=\"action\" value=\"configure_otp\"><input type=submit></form></td></tr>\n";
		$out .= "</table>\n";
		return $out;
	}
	// TOTPSetup will validate if the provided secret and token is valid.  If it is, it will store the details in a table
	// Input 
	// $conn - mySQL database connection (a new table TOTP_Secrets will be created
	// $uid  - the unique ID of the user, typically an integer coming from your user table
	public function TOTPsetup ($conn,$uid) {
		if($this->verifyCode($_POST['secret'],$_POST['otp'])) {
			// Create the table if it doesn't exist
			$this->_create_totp_table($conn);
			// delete any tokens already defined
			$sql = sprintf("DELETE FROM `totp_secrets` WHERE uid = '%s'",
				mysqli_real_escape_string($conn,$uid)
			);
			mysqli_query($conn,$sql);
			// If the token validated, we will store it in the DB
			$sql = sprintf("INSERT INTO `totp_secrets` (uid,secret,datetime,ip_address) VALUES('%s','%s',CURRENT_TIMESTAMP,'%s')",
				mysqli_real_escape_string($conn,$uid),
				mysqli_real_escape_string($conn,$_POST['secret']),
				mysqli_real_escape_string($conn,$_SERVER['REMOTE_ADDR'])
			);
			if(mysqli_query($conn,$sql)) {
				return "SUCCESS : The token was generated successfully.";
			} else {
				return "ERROR : Something went wrong setting up your token.";
			}
		} else {
			return "ERROR : The token did not validate<br>";
		}
	}
	public function TOTPauthenticate($conn,$uid) {
		// Get the secret token for the user
		$sql = sprintf("SELECT secret FROM `totp_secrets` WHERE uid = '%s'",
				mysqli_real_escape_string($conn,$uid));
		$result = mysqli_query($conn,$sql);
		if($result) {
			$row = mysqli_fetch_assoc($result);	
			// validate the code against the id that was provided
			return $this->verifyCode($row['secret'],$_POST['otp']);	
		} else {
			// if TOTP is not configured, then by default we will ALLOW the access.
			return 1;
		}
	}
	// Create a database table if it doesn't exist yet.  We're opting for a new table, because adjusting an existing table schema may break other functions in your application, plus, you should mess with someone else's tables.
	public function _create_totp_table ($conn) {
		if(!mysqli_query($conn,'select 1 from `totp_secrets`')) {
			$sql = "CREATE TABLE `totp_secrets` (
				`uid`		varchar(50) primary key,
				`secret`	varchar(30),
				`datetime`	datetime,
				`ip_address`	varchar(30)
				)";
			if (mysqli_query($conn,$sql)) {
				return 1;
			}
		}
		return 0;
	}
	public function createSecret($secretLength = 16) {
        	$validChars = $this->_getBase32LookupTable();
        	unset($validChars[32]);
        	$secret = '';
        	for ($i = 0; $i < $secretLength; $i++) {
            	$secret .= $validChars[array_rand($validChars)];
        	}
        	return $secret;
	}
	public function generateOTP($key,$interval) {
		// Turn the key into a standard string, no spaces, all upper case
		$key = strtoupper($key);
		$key = preg_replace('/\s+/','',$key);
	
		// decode the key from base32
		$key_decoded = $this->_base32Decode($key);
		// Read the time, and produce the 30 second slice
		$timeSlice = floor(time() / 30) + $interval;
		// Pack the time to binary
		$time = chr(0).chr(0).chr(0).chr(0).pack('N*', $timeSlice);
		// hash the time with the key
		$hmac = hash_hmac('SHA1', $time, $key_decoded, true);
		// get the offset
		$offset = ord(substr($hmac, -1)) & 0x0F;
		// use the offset to get part of the hash
		$hashpart = substr($hmac, $offset, 4);
		// get the first number
		$value = unpack('N', $hashpart);
		$value = $value[1];
	
		// grab the first 32 bits
		$value = $value & 0x7FFFFFFF;
		$value = $value % 1000000;
		return $value;
	}
	public function verifyCode($secret, $code,$discrepency = 1)
	{
		for ($i = -$discrepancy; $i <= $discrepancy; $i++) {
			$calculatedCode = $this->generateOTP($secret, $i);
			if ($calculatedCode == $code ) {
				return true;
			}
		}
		return false;
	}
	
	public function getQRCodeGoogleUrl($name, $secret, $title = null) {
		$urlencoded = urlencode('otpauth://totp/'.$name.'?secret='.$secret.'');
		if(isset($title)) {
			$urlencoded .= urlencode('&issuer='.urlencode($title));
		}
		return 'https://chart.googleapis.com/chart?chs=200x200&chld=M|0&cht=qr&chl='.$urlencoded.'';
	}
	protected function _base32Decode($secret)
	{
		if (empty($secret)) return '';
		$base32chars = $this->_getBase32LookupTable();
		$base32charsFlipped = array_flip($base32chars);
		$paddingCharCount = substr_count($secret, $base32chars[32]);
		$allowedValues = array(6, 4, 3, 1, 0);
		if (!in_array($paddingCharCount, $allowedValues)) return false;
		for ($i = 0; $i < 4; $i++){
			if ($paddingCharCount == $allowedValues[$i] &&
				substr($secret, -($allowedValues[$i])) != str_repeat($base32chars[32], $allowedValues[$i])) return false;
		}
		$secret = str_replace('=','', $secret);
		$secret = str_split($secret);
		$binaryString = "";
		for ($i = 0; $i < count($secret); $i = $i+8) {
			$x = "";
			if (!in_array($secret[$i], $base32chars)) return false;
			for ($j = 0; $j < 8; $j++) {
				$x .= str_pad(base_convert(@$base32charsFlipped[@$secret[$i + $j]], 10, 2), 5, '0', STR_PAD_LEFT);
			}
			$eightBits = str_split($x, 8);
			for ($z = 0; $z < count($eightBits); $z++) {
				$binaryString .= ( ($y = chr(base_convert($eightBits[$z], 2, 10))) || ord($y) == 48 ) ? $y:"";
			}
		}
		return $binaryString;
	}
	protected function _getBase32LookupTable()
	{
		return array(
			'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', //  7
			'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', // 15
			'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', // 23
			'Y', 'Z', '2', '3', '4', '5', '6', '7', // 31
			'='  // padding char
		);
	}
}