<?php
// Use Google Authenticator with AppGini script by Phil Massyn
// For reference, see https://bigprof.com/appgini/help/advanced-topics/custom-limited-access-pages
//
//			  2019-11-17 - enabled self-removal of two factor auth for users. Olaf Nöhring, https://datenbank-projekt.de
#
# How to use
# 1) Copy this script to your hooks folder
# 2) Create a link to this file in hooks/links-home.php and hooks/links-nav.php 
# 3) Update the menu path (see https://bigprof.com/appgini/help/advanced-topics/hooks/folder-contents )
#


define('PREPEND_PATH', '../');
$hooks_dir = dirname(__FILE__);
include("$hooks_dir/../defaultLang.php");
include("$hooks_dir/../language.php");
include("$hooks_dir/../lib.php");
include_once("$hooks_dir/../header.php");


	/* START: grant access to all users who have access to the orders table / non guests */
	// you might need to remove this, if you do not have a table named 'totp_secrets' in yout AppGini application and given access to this to the users
	//	$user_can_access = get_sql_from('totp_secrets');
	//	if(!$user_can_access) exit(error_message('Access denied!', false));
	//alternate:
	//only allow access to this page for members
	if (getLoggedMemberID() == 'guest') exit(error_message('Access denied!', false));
	/* END: grant access to all users who have access to the orders table / non guests*/

require_once "$hooks_dir/GoogleAuthenticatorClass.php";
$ga = new framework_GoogleAuthenticator();
$conn = db_link();
$memberinfo = getMemberInfo();

$removeAuthSubmit = isset($_GET['removeAuthSubmit']) ? $removed = 1 : $removed = 0;
$removeAuth = isset($_GET['removeAuth']) ? makeSafe($_GET['removeAuth']) : 0;
if ($removeAuth == 1) {	
    $primary_key = getLoggedMemberID();
	$sql = "delete from totp_secrets where uid = '" . $primary_key . "';";
	$result = sqlValue($sql);
	echo '<h2>Removed Google Authenticator</h2><p>Your two factor authentication <strong>has been removed</strong>.<br>You may recreate a new token any time to reenable two factor authentication with <a href="'. $_SERVER['PHP_SELF']  .'">Google Authenticator</a>.</p>';
}
else {
	echo '<h2>Setup Google Authenticator</h2>';
if(!isset($_POST['secret']) && !isset($_POST['otp'])) {
        // this is the first page, to present the user with a QR code, and a form to validate the token
        echo $ga->TOTPsetupPage($memberinfo['username'],$host);	// need to find a way to retreive the site name and put it in here.
} elseif(isset($_POST['secret']) && isset($_POST['otp'])) {
	// now we check if the provided token is actually working
	$message = $ga->TOTPsetup($conn,$memberinfo['username']);
	if (preg_match("/ERROR/i", $message)) {
		echo '<div class="alert alert-warning"><i class="glyphicon glyphicon-warning-sign"></i> ';
		echo $message;
		echo '</div>';
		echo '<a href="setup_googleauth.php">Try again</a>';
	} else {
		echo '<div class="alert alert-success"><i class="glyphicon glyphicon-success-sign"></i> ';
		echo $message;
		echo '</div>';
}
}


?>
<h2>Remove Google Authenticator</h2>
<form action="<?php echo $_SERVER["PHP_SELF"]; ?>" method="get">
<div class="removeAuthCheckbox" title="Check the box and submit to remove your need to log in using two factor authentication (if it was set).">
				<input type="checkbox" id="removeAuth" name="removeAuth" value="1">
				<label for="removeAuth">Remove my two factor authentication</label>
			</div><!-- removeAuthCheckbox -->

			<div class="removeAuthButton">
			<span class="btn-group btn-group-sm">
				<button id="removeAuthSubmit" name="removeAuthSubmit" value="1" type="submit" class="btn btn-default" title="Submit">Remove now</button>
			</span>
		</div><!-- removeAuthButton -->
</form>			
<?php
}
include_once("$hooks_dir/../footer.php");
?>